/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.github.liaochong.myexcel.core.SaxExcelReader;
import com.yiring.app.domain.location.LocationTag;
import com.yiring.app.domain.location.LocationTagRepository;
import com.yiring.app.excel.location.LocationTagExportExcel;
import com.yiring.app.excel.location.LocationTagImportExcel;
import com.yiring.app.param.location.LocationTagAddParam;
import com.yiring.app.param.location.LocationTagDeleteParam;
import com.yiring.app.param.location.LocationTagFindParam;
import com.yiring.app.param.location.LocationTagModifyParam;
import com.yiring.app.param.location.zy.ZyLocationTagAddParam;
import com.yiring.app.service.location.LocationTagService;
import com.yiring.app.vo.location.LocationTagVo;
import com.yiring.auth.util.ZyUtil;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.jetbrains.annotations.NotNull;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

/**
 * 定位标签
 * @author LJ-2204
 * @date 2022/4/14
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class LocationTagServiceImpl implements LocationTagService {

    final Integer FACTORY_ID = 100;

    @Resource
    LocationTagRepository locationTagRepository;

    @Override
    public Result<String> addLocationTag(LocationTagAddParam param) {
        if (hasLocationTagInfoByCode(param.getCode())) return Result.no(Status.BAD_REQUEST, "编号已存在");
        LocationTag locationTag = param.transform();
        locationTagRepository.save(locationTag);
        ZyLocationTagAddParam zyLocationTagAddParam = ZyLocationTagAddParam.transform(locationTag);
        List<ZyLocationTagAddParam> of = ListUtil.of(zyLocationTagAddParam);
        new Thread(() -> {
            Result<Serializable> result = ZyUtil.post("/positionApi/api/tag/saveTag", of, 3000);
            if (result.getCode() != 200) throw new RuntimeException(result.getMessage());
        })
            .start();

        return Result.ok();
    }

    @Override
    public Result<PageVo<LocationTagVo>> findLocationTagPage(LocationTagFindParam param) {
        Specification<LocationTag> specification = getLocationTagPageSpecification(param);

        Sort sort = Sort.by(Sort.Order.desc(LocationTag.Fields.createTime));

        if (Objects.nonNull(param.getPageNo()) && Objects.nonNull(param.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(param.getPageNo() - 1, param.getPageSize());
            Page<LocationTag> locationTags = locationTagRepository.findAll(specification, pageable);
            List<LocationTagVo> locationTagVos = LocationTagVo.transforms(locationTags.getContent());
            PageVo<LocationTagVo> page = PageVo.build(locationTagVos, locationTagVos.size());
            return Result.ok(page);
        } else {
            List<LocationTag> locationTags = locationTagRepository.findAll(specification, sort);
            List<LocationTagVo> locationTagVos = LocationTagVo.transforms(locationTags);
            PageVo<LocationTagVo> page = PageVo.build(locationTagVos, locationTagVos.size());
            return Result.ok(page);
        }
    }

    @Override
    public Result<String> deleteLocationTag(LocationTagDeleteParam param) {
        if (ObjectUtil.isNotEmpty(param.getUser())) return Result.no(Status.BAD_REQUEST, "请先解绑人员");
        locationTagRepository.deleteById(param.getId());
        return Result.ok();
    }

    @Override
    public void exportLocationTagInfo(LocationTagFindParam param, HttpServletResponse response) {
        Specification<LocationTag> specification = getLocationTagPageSpecification(param);
        List<LocationTag> locationTags = locationTagRepository.findAll(specification);
        List<LocationTagExportExcel> locationTagExportExcels = LocationTagExportExcel.transforms(locationTags);
        try (
            DefaultStreamExcelBuilder<LocationTagExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(LocationTagExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(locationTagExportExcels);

            String fileName = URLEncoder.encode("定位标签信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出岗位信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> importLocationTagInfo(MultipartFile file) {
        if (file.isEmpty()) throw new RuntimeException("请选择文件");
        List<LocationTagImportExcel> locationTagImportExcels;
        try (InputStream inputStream = file.getInputStream()) {
            locationTagImportExcels =
                SaxExcelReader
                    .of(LocationTagImportExcel.class)
                    .rowFilter(row -> row.getRowNum() > 0)
                    .ignoreBlankRow()
                    .read(inputStream);
        } catch (IOException e) {
            log.info(e.getMessage());
            throw new RuntimeException("文件导入异常");
        }

        List<LocationTag> locationTags = locationTagImportExcels
            .stream()
            .map(LocationTagImportExcel::transform)
            .collect(Collectors.toList());

        locationTags.forEach(locationTag -> {
            locationTagRepository
                .findOne((root, cq, cb) -> {
                    List<Predicate> predicates = new ArrayList<>();

                    if (StrUtil.isNotEmpty(locationTag.getCode())) {
                        predicates.add(cb.equal(root.get(LocationTag.Fields.code), locationTag.getCode()));
                    }

                    return cq.where(predicates.toArray(new Predicate[0])).getRestriction();
                })
                .ifPresent(temp -> locationTag.setId(temp.getId()));
            locationTagRepository.save(locationTag);
        });
        return Result.ok();
    }

    @Override
    public Result<String> modifyLocationTag(LocationTagModifyParam param, IdParam idParam) {
        Optional<LocationTag> locationTagOptional = locationTagRepository.findById(idParam.getId());
        if (locationTagOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "被修改的数据不存在");

        LocationTag locationTag = locationTagOptional.get();

        if (!StrUtil.equals(locationTag.getCode(), param.getCode())) {
            if (hasLocationTagInfoByCode(param.getCode())) return Result.no(Status.BAD_REQUEST, "编号已存在");
        }

        locationTag = param.transform();
        locationTag.setId(idParam.getId());
        locationTagRepository.save(locationTag);

        return Result.ok();
    }

    private boolean hasLocationTagInfoByCode(String code) {
        LocationTag locationTag = LocationTag.builder().code(code).build();
        return locationTagRepository.count(Example.of(locationTag)) > 0;
    }

    @NotNull
    private Specification<LocationTag> getLocationTagPageSpecification(LocationTagFindParam param) {
        return (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(param.getCode())) {
                cb.equal(root.get(LocationTag.Fields.code), param.getCode());
            }

            if (ObjectUtil.isNotEmpty(param.getType())) {
                cb.equal(root.get(LocationTag.Fields.type), param.getType());
            }

            if (ObjectUtil.isNotEmpty(param.getSilent())) {
                cb.equal(root.get(LocationTag.Fields.silent), param.getSilent());
            }

            Order order = cb.desc(root.get(LocationTag.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };
    }
}
